# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class ColonTestCase(RuleTestCase):
    rule_id = 'braces'

    def test_disabled(self):
        conf = 'braces: disable'
        self.check('---\n'
                   'dict1: {}\n'
                   'dict2: { }\n'
                   'dict3: {   a: 1, b}\n'
                   'dict4: {a: 1, b, c: 3 }\n'
                   'dict5: {a: 1, b, c: 3 }\n'
                   'dict6: {  a: 1, b, c: 3 }\n'
                   'dict7: {   a: 1, b, c: 3 }\n', conf)

    def test_min_spaces(self):
        conf = ('braces:\n'
                '  max-spaces-inside: -1\n'
                '  min-spaces-inside: 0\n'
                '  max-spaces-inside-empty: -1\n'
                '  min-spaces-inside-empty: -1\n')
        self.check('---\n'
                   'dict: {}\n', conf)

        conf = ('braces:\n'
                '  max-spaces-inside: -1\n'
                '  min-spaces-inside: 1\n'
                '  max-spaces-inside-empty: -1\n'
                '  min-spaces-inside-empty: -1\n')
        self.check('---\n'
                   'dict: {}\n', conf, problem=(2, 8))
        self.check('---\n'
                   'dict: { }\n', conf)
        self.check('---\n'
                   'dict: {a: 1, b}\n', conf,
                   problem1=(2, 8), problem2=(2, 15))
        self.check('---\n'
                   'dict: { a: 1, b }\n', conf)
        self.check('---\n'
                   'dict: {\n'
                   '  a: 1,\n'
                   '  b\n'
                   '}\n', conf)

        conf = ('braces:\n'
                '  max-spaces-inside: -1\n'
                '  min-spaces-inside: 3\n'
                '  max-spaces-inside-empty: -1\n'
                '  min-spaces-inside-empty: -1\n')
        self.check('---\n'
                   'dict: { a: 1, b }\n', conf,
                   problem1=(2, 9), problem2=(2, 17))
        self.check('---\n'
                   'dict: {   a: 1, b   }\n', conf)

    def test_max_spaces(self):
        conf = ('braces:\n'
                '  max-spaces-inside: 0\n'
                '  min-spaces-inside: -1\n'
                '  max-spaces-inside-empty: -1\n'
                '  min-spaces-inside-empty: -1\n')
        self.check('---\n'
                   'dict: {}\n', conf)
        self.check('---\n'
                   'dict: { }\n', conf, problem=(2, 8))
        self.check('---\n'
                   'dict: {a: 1, b}\n', conf)
        self.check('---\n'
                   'dict: { a: 1, b }\n', conf,
                   problem1=(2, 8), problem2=(2, 16))
        self.check('---\n'
                   'dict: {   a: 1, b   }\n', conf,
                   problem1=(2, 10), problem2=(2, 20))
        self.check('---\n'
                   'dict: {\n'
                   '  a: 1,\n'
                   '  b\n'
                   '}\n', conf)

        conf = ('braces:\n'
                '  max-spaces-inside: 3\n'
                '  min-spaces-inside: -1\n'
                '  max-spaces-inside-empty: -1\n'
                '  min-spaces-inside-empty: -1\n')
        self.check('---\n'
                   'dict: {   a: 1, b   }\n', conf)
        self.check('---\n'
                   'dict: {    a: 1, b     }\n', conf,
                   problem1=(2, 11), problem2=(2, 23))

    def test_min_and_max_spaces(self):
        conf = ('braces:\n'
                '  max-spaces-inside: 0\n'
                '  min-spaces-inside: 0\n'
                '  max-spaces-inside-empty: -1\n'
                '  min-spaces-inside-empty: -1\n')
        self.check('---\n'
                   'dict: {}\n', conf)
        self.check('---\n'
                   'dict: { }\n', conf, problem=(2, 8))
        self.check('---\n'
                   'dict: {   a: 1, b}\n', conf, problem=(2, 10))

        conf = ('braces:\n'
                '  max-spaces-inside: 1\n'
                '  min-spaces-inside: 1\n'
                '  max-spaces-inside-empty: -1\n'
                '  min-spaces-inside-empty: -1\n')
        self.check('---\n'
                   'dict: {a: 1, b, c: 3 }\n', conf, problem=(2, 8))

        conf = ('braces:\n'
                '  max-spaces-inside: 2\n'
                '  min-spaces-inside: 0\n'
                '  max-spaces-inside-empty: -1\n'
                '  min-spaces-inside-empty: -1\n')
        self.check('---\n'
                   'dict: {a: 1, b, c: 3 }\n', conf)
        self.check('---\n'
                   'dict: {  a: 1, b, c: 3 }\n', conf)
        self.check('---\n'
                   'dict: {   a: 1, b, c: 3 }\n', conf, problem=(2, 10))

    def test_min_spaces_empty(self):
        conf = ('braces:\n'
                '  max-spaces-inside: -1\n'
                '  min-spaces-inside: -1\n'
                '  max-spaces-inside-empty: 0\n'
                '  min-spaces-inside-empty: 0\n')
        self.check('---\n'
                   'array: {}\n', conf)

        conf = ('braces:\n'
                '  max-spaces-inside: -1\n'
                '  min-spaces-inside: -1\n'
                '  max-spaces-inside-empty: -1\n'
                '  min-spaces-inside-empty: 1\n')
        self.check('---\n'
                   'array: {}\n', conf, problem=(2, 9))
        self.check('---\n'
                   'array: { }\n', conf)

        conf = ('braces:\n'
                '  max-spaces-inside: -1\n'
                '  min-spaces-inside: -1\n'
                '  max-spaces-inside-empty: -1\n'
                '  min-spaces-inside-empty: 3\n')
        self.check('---\n'
                   'array: {}\n', conf, problem=(2, 9))
        self.check('---\n'
                   'array: {   }\n', conf)

    def test_max_spaces_empty(self):
        conf = ('braces:\n'
                '  max-spaces-inside: -1\n'
                '  min-spaces-inside: -1\n'
                '  max-spaces-inside-empty: 0\n'
                '  min-spaces-inside-empty: -1\n')
        self.check('---\n'
                   'array: {}\n', conf)
        self.check('---\n'
                   'array: { }\n', conf, problem=(2, 9))

        conf = ('braces:\n'
                '  max-spaces-inside: -1\n'
                '  min-spaces-inside: -1\n'
                '  max-spaces-inside-empty: 1\n'
                '  min-spaces-inside-empty: -1\n')
        self.check('---\n'
                   'array: {}\n', conf)
        self.check('---\n'
                   'array: { }\n', conf)
        self.check('---\n'
                   'array: {  }\n', conf, problem=(2, 10))

        conf = ('braces:\n'
                '  max-spaces-inside: -1\n'
                '  min-spaces-inside: -1\n'
                '  max-spaces-inside-empty: 3\n'
                '  min-spaces-inside-empty: -1\n')
        self.check('---\n'
                   'array: {}\n', conf)
        self.check('---\n'
                   'array: {   }\n', conf)
        self.check('---\n'
                   'array: {    }\n', conf, problem=(2, 12))

    def test_min_and_max_spaces_empty(self):
        conf = ('braces:\n'
                '  max-spaces-inside: -1\n'
                '  min-spaces-inside: -1\n'
                '  max-spaces-inside-empty: 2\n'
                '  min-spaces-inside-empty: 1\n')
        self.check('---\n'
                   'array: {}\n', conf, problem=(2, 9))
        self.check('---\n'
                   'array: { }\n', conf)
        self.check('---\n'
                   'array: {  }\n', conf)
        self.check('---\n'
                   'array: {   }\n', conf, problem=(2, 11))

    def test_mixed_empty_nonempty(self):
        conf = ('braces:\n'
                '  max-spaces-inside: -1\n'
                '  min-spaces-inside: 1\n'
                '  max-spaces-inside-empty: 0\n'
                '  min-spaces-inside-empty: 0\n')
        self.check('---\n'
                   'array: { a: 1, b }\n', conf)
        self.check('---\n'
                   'array: {a: 1, b}\n', conf,
                   problem1=(2, 9), problem2=(2, 16))
        self.check('---\n'
                   'array: {}\n', conf)
        self.check('---\n'
                   'array: { }\n', conf,
                   problem1=(2, 9))

        conf = ('braces:\n'
                '  max-spaces-inside: 0\n'
                '  min-spaces-inside: -1\n'
                '  max-spaces-inside-empty: 1\n'
                '  min-spaces-inside-empty: 1\n')
        self.check('---\n'
                   'array: { a: 1, b }\n', conf,
                   problem1=(2, 9), problem2=(2, 17))
        self.check('---\n'
                   'array: {a: 1, b}\n', conf)
        self.check('---\n'
                   'array: {}\n', conf,
                   problem1=(2, 9))
        self.check('---\n'
                   'array: { }\n', conf)

        conf = ('braces:\n'
                '  max-spaces-inside: 2\n'
                '  min-spaces-inside: 1\n'
                '  max-spaces-inside-empty: 1\n'
                '  min-spaces-inside-empty: 1\n')
        self.check('---\n'
                   'array: { a: 1, b  }\n', conf)
        self.check('---\n'
                   'array: {a: 1, b   }\n', conf,
                   problem1=(2, 9), problem2=(2, 18))
        self.check('---\n'
                   'array: {}\n', conf,
                   problem1=(2, 9))
        self.check('---\n'
                   'array: { }\n', conf)
        self.check('---\n'
                   'array: {   }\n', conf,
                   problem1=(2, 11))

        conf = ('braces:\n'
                '  max-spaces-inside: 1\n'
                '  min-spaces-inside: 1\n'
                '  max-spaces-inside-empty: 1\n'
                '  min-spaces-inside-empty: 1\n')
        self.check('---\n'
                   'array: { a: 1, b }\n', conf)
        self.check('---\n'
                   'array: {a: 1, b}\n', conf,
                   problem1=(2, 9), problem2=(2, 16))
        self.check('---\n'
                   'array: {}\n', conf,
                   problem1=(2, 9))
        self.check('---\n'
                   'array: { }\n', conf)



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from setuptools import find_packages, setup

from yamllint import (__author__, __license__,
                      APP_NAME, APP_VERSION, APP_DESCRIPTION)


setup(
    name=APP_NAME,
    version=APP_VERSION,
    author=__author__,
    description=APP_DESCRIPTION.split('\n')[0],
    long_description=APP_DESCRIPTION,
    license=__license__,
    keywords=['yaml', 'lint', 'linter', 'syntax', 'checker'],
    url='https://github.com/adrienverge/yamllint',
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development',
        'Topic :: Software Development :: Debuggers',
        'Topic :: Software Development :: Quality Assurance',
        'Topic :: Software Development :: Testing',
    ],

    packages=find_packages(exclude=['tests', 'tests.*']),
    entry_points={'console_scripts': ['yamllint=yamllint.cli:run']},
    package_data={'yamllint': ['conf/*.yaml']},
    install_requires=['pathspec >=0.5.3', 'pyyaml'],
    test_suite='tests',
)



# -*- coding: utf-8 -*-
# Copyright (C) 2017 Johannes F. Knauf
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class KeyOrderingTestCase(RuleTestCase):
    rule_id = 'key-ordering'

    def test_disabled(self):
        conf = 'key-ordering: disable'
        self.check('---\n'
                   'block mapping:\n'
                   '  secondkey: a\n'
                   '  firstkey: b\n', conf)
        self.check('---\n'
                   'flow mapping:\n'
                   '  {secondkey: a, firstkey: b}\n', conf)
        self.check('---\n'
                   'second: before_first\n'
                   'at: root\n', conf)
        self.check('---\n'
                   'nested but OK:\n'
                   '  second: {first: 1}\n'
                   '  third:\n'
                   '    second: 2\n', conf)

    def test_enabled(self):
        conf = 'key-ordering: enable'
        self.check('---\n'
                   'block mapping:\n'
                   '  secondkey: a\n'
                   '  firstkey: b\n', conf,
                   problem=(4, 3))
        self.check('---\n'
                   'flow mapping:\n'
                   '  {secondkey: a, firstkey: b}\n', conf,
                   problem=(3, 18))
        self.check('---\n'
                   'second: before_first\n'
                   'at: root\n', conf,
                   problem=(3, 1))
        self.check('---\n'
                   'nested but OK:\n'
                   '  second: {first: 1}\n'
                   '  third:\n'
                   '    second: 2\n', conf)

    def test_word_length(self):
        conf = 'key-ordering: enable'
        self.check('---\n'
                   'a: 1\n'
                   'ab: 1\n'
                   'abc: 1\n', conf)
        self.check('---\n'
                   'a: 1\n'
                   'abc: 1\n'
                   'ab: 1\n', conf,
                   problem=(4, 1))

    def test_key_duplicates(self):
        conf = ('key-duplicates: disable\n'
                'key-ordering: enable')
        self.check('---\n'
                   'key: 1\n'
                   'key: 2\n', conf)

    def test_case(self):
        conf = 'key-ordering: enable'
        self.check('---\n'
                   'T-shirt: 1\n'
                   'T-shirts: 2\n'
                   't-shirt: 3\n'
                   't-shirts: 4\n', conf)
        self.check('---\n'
                   'T-shirt: 1\n'
                   't-shirt: 2\n'
                   'T-shirts: 3\n'
                   't-shirts: 4\n', conf,
                   problem=(4, 1))

    def test_accents(self):
        conf = 'key-ordering: enable'
        self.check('---\n'
                   'hair: true\n'
                   'hais: true\n'
                   'haïr: true\n'
                   'haïssable: true\n', conf)
        self.check('---\n'
                   'haïr: true\n'
                   'hais: true\n', conf,
                   problem=(3, 1))
        self.check('---\n'
                   'haïr: true\n'
                   'hais: true\n', conf,
                   problem=(3, 1))

    def test_key_tokens_in_flow_sequences(self):
        conf = 'key-ordering: enable'
        self.check('---\n'
                   '[\n'
                   '  key: value, mappings, in, flow: sequence\n'
                   ']\n', conf)



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import unittest

import yaml

from yamllint.rules.common import get_line_indent


class CommonTestCase(unittest.TestCase):
    def test_get_line_indent(self):
        tokens = list(yaml.scan('a: 1\n'
                                'b:\n'
                                '  - c: [2, 3, {d: 4}]\n'))

        self.assertEqual(tokens[3].value, 'a')
        self.assertEqual(tokens[5].value, '1')
        self.assertEqual(tokens[7].value, 'b')
        self.assertEqual(tokens[13].value, 'c')
        self.assertEqual(tokens[16].value, '2')
        self.assertEqual(tokens[18].value, '3')
        self.assertEqual(tokens[22].value, 'd')
        self.assertEqual(tokens[24].value, '4')

        for i in (3, 5):
            self.assertEqual(get_line_indent(tokens[i]), 0)
        for i in (7,):
            self.assertEqual(get_line_indent(tokens[i]), 0)
        for i in (13, 16, 18, 22, 24):
            self.assertEqual(get_line_indent(tokens[i]), 2)



# -*- coding: utf-8 -*-
# Copyright (C) 2016 Adrien Vergé
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from tests.common import RuleTestCase


class DocumentStartTestCase(RuleTestCase):
    rule_id = 'document-start'

    def test_disabled(self):
        conf = 'document-start: disable'
        self.check('', conf)
        self.check('key: val\n', conf)
        self.check('---\n'
                   'key: val\n', conf)

    def test_required(self):
        conf = ('document-start: {present: true}\n'
                'empty-lines: disable\n')
        self.check('', conf)
        self.check('\n', conf)
        self.check('key: val\n', conf, problem=(1, 1))
        self.check('\n'
                   '\n'
                   'key: val\n', conf, problem=(3, 1))
        self.check('---\n'
                   'key: val\n', conf)
        self.check('\n'
                   '\n'
                   '---\n'
                   'key: val\n', conf)

    def test_forbidden(self):
        conf = ('document-start: {present: false}\n'
                'empty-lines: disable\n')
        self.check('', conf)
        self.check('key: val\n', conf)
        self.check('\n'
                   '\n'
                   'key: val\n', conf)
        self.check('---\n'
                   'key: val\n', conf, problem=(1, 1))
        self.check('\n'
                   '\n'
                   '---\n'
                   'key: val\n', conf, problem=(3, 1))
        self.check('first: document\n'
                   '---\n'
                   'key: val\n', conf, problem=(2, 1))

    def test_multiple_documents(self):
        conf = 'document-start: {present: true}'
        self.check('---\n'
                   'first: document\n'
                   '...\n'
                   '---\n'
                   'second: document\n'
                   '...\n'
                   '---\n'
                   'third: document\n', conf)
        self.check('---\n'
                   'first: document\n'
                   '---\n'
                   'second: document\n'
                   '---\n'
                   'third: document\n', conf)
        self.check('---\n'
                   'first: document\n'
                   '...\n'
                   'second: document\n'
                   '---\n'
                   'third: document\n', conf, problem=(4, 1, 'syntax'))

    def test_directives(self):
        conf = 'document-start: {present: true}'
        self.check('%YAML 1.2\n'
                   '---\n'
                   'doc: ument\n'
                   '...\n', conf)
        self.check('%YAML 1.2\n'
                   '%TAG ! tag:clarkevans.com,2002:\n'
                   '---\n'
                   'doc: ument\n'
                   '...\n', conf)
        self.check('---\n'
                   'doc: 1\n'
                   '...\n'
                   '%YAML 1.2\n'
                   '---\n'
                   'doc: 2\n'
                   '...\n', conf)
